from uuid import UUID
from typing import Optional, List, Dict, Any

from .base import BaseClient
from .types import Criteria

class CriteriaClient():
    def __init__(self, base: BaseClient):
        self._base = base

    def upsert(self,
               client_uuid: Optional[UUID | str],
               criteria_uuid: Optional[UUID | str],
               criteria_name: str,
               criteria_prompt_id: str,
               criteria_group_uuid: UUID | str,
               criteria_parent_uuid: Optional[UUID | str] = None,
               criteria_examples: Optional[List[str]] = None,
               criteria_iterate: bool = False,
               criteria_weight: float = 1.0,
               criteria_actor: str = "default",
               criteria_description: Optional[str] = None) -> UUID:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'criteria_uuid': str(criteria_uuid) if criteria_uuid else None,
            'criteria_name': criteria_name,
            'criteria_prompt_id': criteria_prompt_id,
            'criteria_parent_uuid': str(criteria_parent_uuid) if criteria_parent_uuid else None,
            'criteria_examples': criteria_examples or [],
            'criteria_iterate': criteria_iterate,
            'criteria_weight': criteria_weight,
            'criteria_actor': criteria_actor,
            'criteria_description': criteria_description or "",
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('criteria', 'POST', body=body)
        return UUID(resp['criteria_uuid'])

    def delete(self, criteria_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_uuid': str(criteria_uuid)
        }
        resp = self._base.make_request('criteria', 'DELETE', params=params)
        return resp['success']

    def get_by_uuid(self, criteria_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> Criteria:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_uuid': str(criteria_uuid)
        }
        resp = self._base.make_request('criteria', 'GET', params=params)
        return self._parse_criteria(resp)

    def get_list(self, client_uuid: Optional[UUID | str] = None) -> List[Criteria]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {'client_uuid': client_uuid}
        resp = self._base.make_request('criteria', 'GET', params=params)
        print(resp)
        return [self._parse_criteria(c) for c in resp]

    def _parse_criteria(self, data: Dict[str, Any]) -> Criteria:
        return Criteria(
            criteria_uuid=data['criteria_uuid'],
            criteria_name=data['criteria_name'],
            criteria_prompt_id=data['criteria_prompt_id'],
            criteria_parent_uuid=data.get('criteria_parent_uuid'),
            criteria_examples=data['criteria_examples'],
            criteria_iterate=data['criteria_iterate'],
            criteria_weight=data['criteria_weight'],
            criteria_actor=data['criteria_actor'],
            criteria_description=data['criteria_description'],
        )



from typing import Optional, List
from uuid import UUID
from .types import Department

from neurohub.base import BaseClient


class Departments():
    def __init__(self, base: BaseClient):
        self._base = base

    def upsert(self, department_name: str, department_uuid: Optional[str | UUID] = None,
               latitude: Optional[float] = None, longitude: Optional[float] = None, client_uuid: Optional[str] = None) -> UUID:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'department_uuid': department_uuid,
            'department_name': department_name,
            'latitude': latitude,
            'longitude': longitude
        }
        resp = self._base.make_request('department', 'POST', body=body)
        return UUID(resp['department_uuid'])

    def delete(self, department_uuid: str | UUID, client_uuid: Optional[str] = None) -> bool:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'department_uuid': department_uuid
        }
        resp = self._base.make_request('department', 'DELETE', params=params)
        return resp['success']

    def get_by_uuid(self, department_uuid: str, client_uuid: Optional[str] = None):
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'department_uuid': department_uuid
        }
        resp = self._base.make_request('department', 'GET', params=params)
        return self._parse_json_department(resp)

    def get_list(self, client_uuid: Optional[str] = None) -> List[Department]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {'client_uuid': client_uuid}
        resp = self._base.make_request('department', 'GET', params=params)
        parsed_resp = [self._parse_json_department(dept) for dept in resp]
        return parsed_resp

    def _parse_json_department(self, dict):
        return Department(
            department_uuid=str(dict['department_uuid']),
            department_name=dict['department_name'],
            client_uuid=str(dict['client_uuid']),
            latitude=dict['latitude'],
            longitude=dict['longitude']
        )



from typing import Literal, Optional
from neurohub.base import BaseClient
from uuid import UUID
from typing import IO


CallType = Literal['Outgoing', 'Incoming', 'IncomingRedirection', 'Callback']


class Files():
    def __init__(self, base: BaseClient):
        self._base = base
    def post(self, manager_uuid: UUID | str, file_content: bytes | IO, checklist_uuid: UUID | str, call_id: str, call_type: Optional[CallType] = None, client_uuid: Optional[UUID | str] = None):
        if not self._base.s3_client:
            raise RuntimeError("S3 client is not configured. Please provide AWS credentials when initializing the client.")
        client_uuid = self._base._handle_client_uuid(client_uuid)

        file_name = f"{call_id}.mp3"
        s3_key = f"input/{file_name}"

        self._base.s3_client.put_object(
            Bucket=self._base.s3_bucket,
            Key=s3_key,
            Body=file_content
        )

        # TODO: add audio_channels and file_params to args
        body = {
            'client_uuid': client_uuid,
            'manager_uuid': manager_uuid,
            'file_name': file_name,
            'checklist_uuid': str(checklist_uuid),
            'call_type': call_type
        }
        resp = self._base.make_request('file', 'POST', body=body)
        return UUID(resp['file_uuid'])



from uuid import UUID
from typing import Optional

from neurohub.base import BaseClient


class Managers():
    def __init__(self, base: BaseClient):
        self.base = base
    def upsert(self, manager_name: str, department_uuid: UUID | str, manager_uuid: Optional[UUID | str] = None, client_uuid: Optional[UUID | str] = None):
        client_uuid = self.base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'manager_uuid': manager_uuid,
            'manager_name': manager_name,
            'department_uuid': department_uuid
        }
        resp = self.base.make_request('manager', 'POST', body)
        return UUID(resp['manager_uuid'])
    def delete(self, manager_uuid: UUID | str, client_uuid: Optional[UUID | str]) -> bool:
        client_uuid = self.base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'manager_uuid': manager_uuid
        }
        resp = self.base.make_request('manager', 'DELETE', params=params)
        return resp['success']


    def get_by_uuid(self, manager_uuid: UUID | str, client_uuid: Optional[UUID | str] = None):
        """Get a single manager by UUID"""
        client_uuid = self.base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': str(client_uuid),
            'manager_uuid': str(manager_uuid)
        }
        resp = self.base.make_request('manager', 'GET', params=params)
        return resp

    def get_list(self, client_uuid: Optional[UUID | str] = None):
        """Get all managers for a client"""
        client_uuid = self.base._handle_client_uuid(client_uuid)
        params = {'client_uuid': client_uuid}
        resp = self.base.make_request('manager', 'GET', params=params)
        return resp



from uuid import UUID
from typing import Optional, List, Dict, Any

from .base import BaseClient
from .types import CriteriaGroup
from neurohub.types import CriteriaGroupCriteriaLink

class CriteriaGroups():
    def __init__(self, base: BaseClient):
        self._base = base

    def upsert(self,
               client_uuid: Optional[UUID | str],
               criteria_group_uuid: Optional[UUID | str],
               criteria_group_name: str,
               criteria_group_prompt_id: str,
               parent_criteria_group_uuid: Optional[UUID | str] = None,
               criteria_group_weight: float = 1.0,
               criteria_group_rules: Optional[Dict[str, Any]] = None,
               criteria_group_type: str = "default") -> UUID:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid) if criteria_group_uuid else None,
            'criteria_group_name': criteria_group_name,
            'criteria_group_prompt_id': criteria_group_prompt_id,
            'parent_criteria_group_uuid': str(parent_criteria_group_uuid) if parent_criteria_group_uuid else None,
            'criteria_group_weight': criteria_group_weight,
            'criteria_group_rules': criteria_group_rules or {},
            'criteria_group_type': criteria_group_type
        }
        resp = self._base.make_request('criteria-group', 'POST', body=body)
        return UUID(resp['criteria_group_uuid'])

    def delete(self, criteria_group_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('criteria-group', 'DELETE', params=params)
        return resp['success']

    def get_by_uuid(self, criteria_group_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> CriteriaGroup:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('criteria-group', 'GET', params=params)
        print('Resp', resp)
        return self._parse_criteria_group(resp)

    def get_list(self, client_uuid: Optional[UUID | str] = None) -> List[CriteriaGroup]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {'client_uuid': client_uuid}
        resp = self._base.make_request('criteria-group', 'GET', params=params)
        print('Resp', resp)
        return [self._parse_criteria_group(group) for group in resp]

    def _parse_criteria_group(self, data: Dict[str, Any]) -> CriteriaGroup:
        """Strict parsing into CriteriaGroup TypedDict"""
        return CriteriaGroup(
            criteria_group_uuid=data['criteria_group_uuid'],
            criteria_group_name=data['criteria_group_name'],
            criteria_group_prompt_id=data['criteria_group_prompt_id'],
            parent_criteria_group_uuid=data.get('parent_criteria_group_uuid'),
            criteria_group_weight=data['criteria_group_weight'],
            criteria_group_rules=data['criteria_group_rules'],
            criteria_group_type=data['criteria_group_type'],
            # criteria=[self._parse_criteria(c) for c in data['criteria']]
        )
    def add_criteria(self, criteria_group_uuid: UUID | str, criteria_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        """POST /criteria-group-criteria"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid),
            'criteria_uuid': str(criteria_uuid)
        }
        resp = self._base.make_request('criteria-group-criteria', 'POST', body=body)
        return resp['success']

    def remove_criteria(self, criteria_group_uuid: UUID | str, criteria_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        """DELETE /criteria-group-criteria"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid),
            'criteria_uuid': str(criteria_uuid)
        }
        resp = self._base.make_request('criteria-group-criteria', 'DELETE', params=params)
        return resp['success']

    def list_criteria(self, criteria_group_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> List[CriteriaGroupCriteriaLink]:
        """GET /criteria-group-criteria"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('criteria-group-criteria', 'GET', params=params)
        return [self._parse_criteria_link(item) for item in resp]

    def _parse_criteria_link(self, data: dict) -> CriteriaGroupCriteriaLink:
        return CriteriaGroupCriteriaLink(
            criteria_group_uuid=data['criteria_group_uuid'],
            criteria_group_name=data['criteria_group_name'],
            criteria_uuid=data['criteria_uuid'],
            criteria_name=data['criteria_name']
        )



from neurohub.base import BaseClient
from typing import Optional
from uuid import UUID

from neurohub.checklists import Checklists
from neurohub.clients import Clients
from neurohub.criteria import CriteriaClient
from neurohub.criteria_groups import CriteriaGroups
from neurohub.criteria_result import CriteriaResult
from neurohub.departments import Departments
from neurohub.managers import Managers
from neurohub.file import Files


class RootAPI():
    def __init__(self, secret_key: str, client_uuid: Optional[UUID],
        s3_bucket: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None):
        self.base = BaseClient(secret_key, client_uuid, s3_bucket, aws_access_key_id, aws_secret_access_key)

    @property
    def client_uuid(self) -> Optional[UUID]:
        return self.base.client_uuid
    @client_uuid.setter
    def client_uuid(self, value: UUID):
        self.base.client_uuid = value

    @property
    def managers(self):
        return Managers(self.base)
    @property
    def clients(self):
        return Clients(self.base)
    @property
    def departments(self):
        return Departments(self.base)
    @property
    def checklists(self):
        return Checklists(self.base)
    @property
    def criteria_groups(self):
        return CriteriaGroups(self.base)
    @property
    def files(self):
        return Files(self.base)
    @property
    def criteria(self):
        return CriteriaClient(self.base)
    @property
    def criteria_result(self):
        return CriteriaResult(self.base)



from uuid import UUID
from typing import Optional, List, Dict, Any

from neurohub.base import BaseClient
from neurohub.types import ChecklistCriteriaGroupLink


class Checklists():
    def __init__(self, base: BaseClient):
        self._base = base

    def upsert(self, client_uuid: Optional[UUID | str], checklist_uuid: Optional[UUID | str], checklist_name: str) -> UUID | str:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid) if checklist_uuid else None,
            'checklist_name': checklist_name
        }
        resp = self._base.make_request('checklist', 'POST', body=body)
        return UUID(resp['checklist_uuid'])

    def delete(self, checklist_uuid: UUID | str, client_uuid: Optional[UUID | str]) -> bool:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid)
        }
        resp = self._base.make_request('checklist', 'DELETE', params=params)
        return resp['success']

    def get_by_uuid(self, checklist_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> Dict[str, Any]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid)
        }
        resp = self._base.make_request('checklist', 'GET', params=params)
        return resp

    def get_by_client(self, client_uuid: Optional[UUID | str] = None) -> List[Dict[str, Any]]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {'client_uuid': client_uuid}
        resp = self._base.make_request('checklist', 'GET', params=params)
        return resp


    def link_criteria_group(self, checklist_uuid: UUID | str, criteria_group_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        """POST /checklist-criteria-group"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        body = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid),
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('checklist-criteria-group', 'POST', body=body)
        return resp['success']

    def unlink_criteria_group(self, checklist_uuid: UUID | str, criteria_group_uuid: UUID | str, client_uuid: Optional[UUID | str] = None) -> bool:
        """DELETE /checklist-criteria-group"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid),
            'criteria_group_uuid': str(criteria_group_uuid)
        }
        resp = self._base.make_request('checklist-criteria-group', 'DELETE', params=params)
        return resp['success']

    def get_linked_criteria_groups(self, checklist_uuid: UUID | str, client_uuid: Optional[UUID | str] = None):
        """GET /checklist-criteria-group"""
        client_uuid = self._base._handle_client_uuid(client_uuid)
        params = {
            'client_uuid': client_uuid,
            'checklist_uuid': str(checklist_uuid)
        }
        resp = self._base.make_request('checklist-criteria-group', 'GET', params=params)
        return [self._parse_link(item) for item in resp]
    def _parse_link(self, data: dict) -> ChecklistCriteriaGroupLink:
        return ChecklistCriteriaGroupLink(
            checklist_uuid=data['checklist_uuid'],
            checklist_name=data['checklist_name'],
            criteria_group_uuid=data['criteria_group_uuid'],
            criteria_group_name=data['criteria_group_name']
        )



from uuid import UUID
from typing import Any, Optional, List
from .types import CriteriaResultQA, CriteriaResultEntity, CriteriaResultAnalysis
from .base import BaseClient

class CriteriaResult():
    def __init__(self, base: BaseClient):
        self._base = base

    def _validate_params(self, file_uuid: Optional[Any], criteria_result_uuid: Optional[Any]):
        if not file_uuid and not criteria_result_uuid:
            raise ValueError('Either file_uuid or criteria_result_qa_uuid must be provided')

    def get_qa(self,
               file_uuid: Optional[UUID | str] = None,
               criteria_result_qa_uuid: Optional[UUID | str] = None,
               client_uuid: Optional[UUID | str] = None) -> List[CriteriaResultQA]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        self._validate_params(file_uuid, criteria_result_qa_uuid)
        params = {
            'client_uuid': client_uuid,
            'file_uuid': str(file_uuid) if file_uuid else None,
            'criteria_result_qa_uuid': str(criteria_result_qa_uuid) if criteria_result_qa_uuid else None
        }
        resp = self._base.make_request('criteria-result-qa', method='GET', params=params)
        return [self._parse_qa_result(item) for item in resp]

    def get_entity(self,
                   file_uuid: Optional[UUID | str] = None,
                   criteria_result_entity_uuid: Optional[UUID | str] = None,
                   client_uuid: Optional[UUID | str] = None) -> List[CriteriaResultEntity]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        self._validate_params(criteria_result_entity_uuid, file_uuid)
        params = {
            'client_uuid': client_uuid,
            'file_uuid': str(file_uuid) if file_uuid else None,
            'criteria_result_entity_uuid': str(criteria_result_entity_uuid) if criteria_result_entity_uuid else None
        }
        resp = self._base.make_request('criteria-result-entity', method='GET', params=params)
        return [self._parse_entity_result(item) for item in resp]

    def get_analysis(self,
                     file_uuid: Optional[UUID | str] = None,
                     criteria_result_analysis_uuid: Optional[UUID | str] = None,
                     client_uuid: Optional[UUID | str] = None) -> List[CriteriaResultAnalysis]:
        client_uuid = self._base._handle_client_uuid(client_uuid)
        self._validate_params(file_uuid, criteria_result_analysis_uuid)
        params = {
            'client_uuid': client_uuid,
            'file_uuid': str(file_uuid) if file_uuid else None,
            'criteria_result_analysis_uuid': str(criteria_result_analysis_uuid) if criteria_result_analysis_uuid else None
        }
        resp = self._base.make_request('criteria-result-analysis', method='GET', params=params)
        return [self._parse_analysis_result(item) for item in resp]

    def _parse_qa_result(self, data: dict) -> CriteriaResultQA:
        return CriteriaResultQA(
            criteria_result_qa_uuid=data['criteria_result_qa_uuid'],
            file_uuid=data['file_uuid'],
            criteria_uuid=data['criteria_uuid'],
            situation_id=data['situation_id'],
            criteria_score=data['criteria_score'],
            qa_time_1=data['qa_time_1'],
            qa_time_2=data['qa_time_2'],
            qa_time_3=data['qa_time_3'],
            criteria=data['criteria']
        )

    def _parse_entity_result(self, data: dict) -> CriteriaResultEntity:
        return CriteriaResultEntity(
            criteria_result_entity_uuid=data['criteria_result_entity_uuid'],
            file_uuid=data['file_uuid'],
            criteria_uuid=data['criteria_uuid'],
            criteria_value=data['criteria_value'],
            entity_time=data['entity_time'],
            criteria=data['criteria'],
            criteria_clasification_list=data['criteria_clasification_list']
        )

    def _parse_analysis_result(self, data: dict) -> CriteriaResultAnalysis:
        return CriteriaResultAnalysis(
            criteria_result_analysis_uuid=data['criteria_result_analysis_uuid'],
            file_uuid=data['file_uuid'],
            criteria_uuid=data['criteria_uuid'],
            criteria_result=data['criteria_result'],
            analysis_evidence=data['analysis_evidence'],
            criteria=data['criteria'],
            classifications=data['classifications']
        )



from typing import Any, Dict, Optional
from uuid import UUID
import httpx
import boto3

from neurohub.errors import MissingClientUUID
class BaseClient():
    base_url = 'https://v2.api.voiceai.neuro-hub.ru/'
    def __init__(self, secret_key: str, client_uuid: Optional[UUID],
        s3_bucket: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None):
        self.secret_key = secret_key
        self.headers = {
            "Authorization": f"Bearer {secret_key}",
            "Content-Type": "application/json"
        }
        self.client = httpx.Client(headers=self.headers, base_url=self.base_url)
        self.client_uuid = client_uuid


        # Initialize S3 client if credentials provided
        self.s3_bucket = s3_bucket
        if s3_bucket and aws_access_key_id and aws_secret_access_key:
            self.s3_client = boto3.client(
                's3',
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                endpoint_url='https://storage.yandexcloud.net/'
            )
        else:
            self.s3_client = None

    def _transform_body(self, body: Dict[str, Any]):
        result = {}
        for key, value in body.items():
            if value is None:
                continue
            if isinstance(value, UUID):
                result[key] = str(value)
                continue
            result[key] = value
        return result


    def make_request(self, endpoint: str, method: str, body: Optional[Dict[str, Any]]=None, params=None):
        if method == 'GET':
            if params:
                params = self._transform_body(params)
            response = self.client.get(endpoint, params=params)
        elif method == 'DELETE':
            response = self.client.delete(endpoint, params=params)
        else:
            if not body:
                raise ValueError('Provide body when making POST request')
            transformed_body = self._transform_body(body)
            response = self.client.post(endpoint, json=transformed_body)
        # TODO: custom exception handling
        response.raise_for_status()
        resp_body = response.json()
        return resp_body

    def _handle_client_uuid(self, arg: Optional[str | UUID]):
        if arg:
            return str(arg)
        if self.client_uuid:
            return str(self.client_uuid)
        raise MissingClientUUID()



from uuid import UUID
from typing import Optional
from .base import BaseClient


class Clients():
    def __init__(self, base: BaseClient):
        self._base = base
    def upsert(self, client_uuid: Optional[UUID], client_name: str):
        params = {
            'client_uuid': None if not client_uuid else str(client_uuid),
            'client_name': client_name
        }
        resp = self._base.make_request('client', 'POST', params=params)
        return UUID(resp['client_uuid'])



from typing import Optional
from uuid import UUID


class MissingClientUUID(Exception):
    def __init__(self):
        super().__init__()
        self.message = 'Client UUID is missing. Either pass it as argument or in constructor of the client.'

def handle_client_uuid(class_uuid: Optional[UUID], arg_uuid: Optional[UUID]):
    if arg_uuid:
        return arg_uuid
    if class_uuid:
        return class_uuid
    raise MissingClientUUID()



from typing import TypedDict, List, Optional, Dict
from typing import Any

class Client(TypedDict):
    client_uuid: str
    client_name: str

class ChecklistSummary(TypedDict):
    checklist_uuid: str
    checklist_name: str
    client_uuid: str

class Criteria(TypedDict):
    criteria_uuid: str
    criteria_name: str
    criteria_prompt_id: str
    criteria_parent_uuid: Optional[str]
    criteria_examples: List[str]
    criteria_iterate: bool
    criteria_weight: float
    criteria_actor: str
    criteria_description: str

class CriteriaGroup(TypedDict):
    criteria_group_uuid: str
    criteria_group_name: str
    criteria_group_prompt_id: str
    parent_criteria_group_uuid: Optional[str]
    criteria_group_weight: float
    criteria_group_rules: Dict[str, Any]
    criteria_group_type: str
    # criteria: List[Criteria]

class ChecklistDetailed(ChecklistSummary):
    criteria_group: List[CriteriaGroup]

class ManagerDepartment(TypedDict):
    department_name: str
    client_uuid: str

class Manager(TypedDict):
    manager_uuid: str
    manager_name: str
    department_uuid: str
    department: ManagerDepartment

class Department(TypedDict):
    department_uuid: str
    department_name: str
    client_uuid: str
    latitude: float
    longitude: float

class CriteriaResultQA(TypedDict):
    criteria_result_qa_uuid: str
    file_uuid: str
    criteria_uuid: str
    situation_id: str
    criteria_score: float
    qa_time_1: str
    qa_time_2: str
    qa_time_3: str
    criteria: Dict[str, str]  # {name: ..., prompt_id: ...}

class CriteriaResultEntity(TypedDict):
    criteria_result_entity_uuid: str
    file_uuid: str
    criteria_uuid: str
    criteria_value: str
    entity_time: str
    criteria: Dict[str, str]
    criteria_clasification_list: List[Dict[str, Any]]  # Expand this if needed

class CriteriaClasificationGroup(TypedDict):
    criteria_clasification_list_group_uuid: str
    criteria_clasification_list_group_name: str

class CriteriaClasificationItem(TypedDict):
    criteria_clasification_list_uuid: str
    criteria_clasification_list_value: str
    criteria_clasification_list_group: CriteriaClasificationGroup

class CriteriaResultAnalysis(TypedDict):
    criteria_result_analysis_uuid: str
    file_uuid: str
    criteria_uuid: str
    criteria_result: str
    analysis_evidence: str
    criteria: Dict[str, str]
    classifications: List[CriteriaClasificationItem]

class ChecklistCriteriaGroupLink(TypedDict):
    checklist_uuid: str
    checklist_name: str
    criteria_group_uuid: str
    criteria_group_name: str

class CriteriaGroupCriteriaLink(TypedDict):
    criteria_group_uuid: str
    criteria_group_name: str
    criteria_uuid: str
    criteria_name: str
